/***************************************************************************
 *** news_article.pmod
 ***
 *** based on news_article.cpp from Fred, <fred@ceylana.com>
 *** ported to pike by Niels van Sluis, <n.vansluis@amc.nl>
 *** Tue Aug 13 2002, newsobjects-0.9.3
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define false 	0
#define	true	1

class news_article {

  string class_id = "news_article";
  private int pnumber;
  private mapping pheaders = ([]);
  private string pbody;

  void create(void|string temp, void|string subject, void|string author,
              void|object date, void|string id, void|string references,
              void|int bytes, void|int lines, void|string body) {
    news_article(temp,subject,author,date,id,references,bytes,lines,body);
  }

  void news_article (mixed temp,void|string subject, void|string author,
              void|object date, void|string id, void|string references,
              void|int bytes, void|int lines, void|string body) {

    if(stringp(temp)) {
    //
    // news_article(string temp);
    //
    // This constructor is for us with a string of output directly from the
    // xover command.
    //

      init(temp);
      return;
    }

    if(stringp(subject)) {
    //
    // news_article(int number, string subject, string author, news_date date,
    //               string id, string references, double bytes, int lines,
    //               string body = "")
    //
    // This constructor is for use when all of the attributes are already in
    // seperate variables.
    //
        int number = temp;
        body = "";

        setNumber(number);
        setSubject(subject);
        setFrom(author);
        setId(id);
        setReferences(references);
        setBytes(bytes);
        setLines(lines);
        setBody(body);
        setDate(date);
    }

    //
    // news_article();
    //
    // Default constructor creates an empty object.
    //
  
    return;
  }

  int setNumber(int number) {
  //
  // Sets the article number using an int.
  //

    pnumber = number;
    return true;
  }

  int setId(string id) {
  //
  // Set the message id using a string.
  //

    pheaders["Message-ID"] = id;
    return true;
  }

  int setSubject(string subject) {
  //
  // Set the subject using a string.
  //

    pheaders["Subject"] = subject;
    return true;
  }

  int setFrom(string from) {
  //
  // Set the from field using a string.
  //

    pheaders["From"] = from;
    return true;
  }

  int setBytes(int bytes) {
  //
  // Set the number of bytes using a double.
  //

    string temp;
    temp = (string)bytes;
    pheaders["Bytes"] = temp;
    return true;
  }

  int setReferences(string references) {
  //
  // Set the references header using a string.
  //

    pheaders["References"] = references;
    return true;
  }

  int getNumber() {
  //
  // Returns the article number as an int
  //

    return pnumber;
  }

  int getBytes() {
  //
  // Returns the number of bytes as a double
  //

    return (int)pheaders["Bytes"];
  }

  string getFrom() {
  //
  // Returns the from field as a string.
  //

    return pheaders["From"];
  }

  string getId() {
  //
  // Returns the message id as a string.
  //

    return pheaders["Message-ID"];
  }

  int getLines() {
  //
  // Returns the number of lines as an int
  //

    return (int)pheaders["Lines"];
  }

  string getReferences() {
  //
  // Returns the references header as a string.
  //

    return pheaders["References"];
  }

  string getSubject() {
  //
  // returns the subject as a string.
  //

    return pheaders["Subject"];
  }

  int setBody(string body) {
  //
  // Set the article body using a string
  //

    pbody = body;
    return true;
  }

  string getBody() {
  // 
  // Returns the article body as a string.
  //
 
    return pbody;
  }

  int saveBody(string path) {
  //
  // Save the body of the article to a file indicated by path.
  //

    if(strlen(pbody) == 0) return false;
    Stdio.write_file(path,pbody);
    return true;
  }

  int saveArticle(string path) {
  //
  // Save the entire article including headers to a file indicated by path.
  //

    object o = Stdio.FILE();
    o->open(path,"CW");
    foreach(indices(pheaders),string line) {
      o->write("%s: %s\n",line,pheaders[line]);
    }
    o->write("\n");

    o->write(pbody);
    o->close();
    destruct(o);
    return true;
  }

  int setDate(string|object date) {
    if(objectp(date)) {
    //
    // Set the article date with a news_date object.
    //

      pheaders["Date"] = date->rfcOutput();
      return true;
    }

    if(stringp(date)) {
    //
    // setDate(string temp);
    //
    // Sets the date with a string formatted to rfc 977 specs.
    //

      pheaders["Date"] = date;
      return true;
    }
  }

  object getDate() {
  //
  // Returns a news_date object containing the article date.
  //

    object date = .news_date.news_date(pheaders["Date"]);
    return date;
  }

  int init(string temp) {
  //
  // This is the same as calling the constructor with an rfc formatted string.
  //

    setDate("1 Jan 1970 01:01:01 GMT"); // fix for missing dates

    string buf = temp;
    buf/="\t";

    setNumber(buf[0]);
    setSubject((string)buf[1]);
    setFrom((string)buf[2]);
    object mydate = .news_date.news_date((string)buf[3]);
    setDate(mydate);
    setId((string)buf[4]);
    setReferences((string)buf[5]);
    setBytes((int)buf[6]);
    setLines((int)buf[7]);

    return true;
  }

  int setLines(int|string lines) {
  //
  // Set the number of lines using a string or int.
  //
  // setLines(int lines);
  // setLines(string lines);

    pheaders["Lines"] = (int)lines;
    return true;
  }

  int setHeader(string a, void|string b) {
    if(stringp(b)) {
    //
    // setHeader(string header, string text);
    //
    // Set an arbitrary article header using the name of the header and the
    // textual contents of the header.
    //

      string header = a;
      string text = b;

      pheaders[header] = text;
      return true;
    }

    if(stringp(a) && !stringp(b)) {
    //
    // setHeader(string text);
    // 
    // Set an arbitrary article header with a string extracted from an article
    // as "Subject: this is the subject". 
    
      string text = a;
      string c,d;
      sscanf(text,"%s: %s",c,d);
      pheaders[c] = d;
      return true;
    }
  }

  string getAuthor() {
  //
  // Returns the name of the author if present in the from field.
  // Otherwise returns the part of the email address before @ .
  //

    string from = getFrom();
    string rvalue;

    if(sscanf(from,"%*s(%s)%*s", rvalue) == 3) {
      return rvalue;
    }

    if(sscanf(from,"%s<%*s",rvalue) == 2) {
      return rvalue;
    }

    sscanf(from,"%s@%*s",rvalue);
    return rvalue;
  }

  string getEmail() {
    string from = getFrom();
    string rvalue;

    if(sscanf(from,"%*s<%s>%*s",rvalue) == 3 ) {
      return rvalue;
    }

    if(sscanf(from,"<%s>%*s",rvalue) == 2 ) {
      return rvalue;
    }

    if(sscanf(from,"%s(%*s",rvalue) == 2) {
      return rvalue;
    }
   
    return from;
  }
};
