/***************************************************************************
 *** news_connection.pmod
 ***
 *** based on news_connection.cpp from Fred, <fred@ceylana.com>
 *** ported to pike by Niels van Sluis, <n.vansluis@amc.nl>
 *** Tue Aug 15 2002, newsobjects-0.9.3
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define false 	0
#define	true	1

class news_connection {
  string class_id = "news_connection";
  private string presponse;
  private string perror;
  private string pcurrentGroup;
  private string psavePath;
  private int puseGroupName;
  private object std;
  private int pdebug;

  void create(void|string|object a, void|int b) {
    std = .std_connection.std_connection();
    news_connection(a,b);
  }

  void destroy() {
    //
    // if still connected, close connection when object no longer needed
    //

    if(std->isConnected()) {
      quit();
    }
  }

  void news_connection(void|string|object a, void|int b) {

    if(objectp(a)) { // Houston, we've got an object
    //
    // news_connection(news_server server);
    //
    // This constructor take a news_server object as an argument
    //

      object server = a;
      std->setHost(server->getHost());
      std->setPort(server->getPort());
      std->connectHost();
      if(std->isConnected()) {
        presponse = std->receive();
        if(server->getPass() != "") {
          authenticate(server->getUser(),server->getPass());
        }
      }
    }

    if(stringp(a)) {
    //
    // news_connection(string host, int port);
    //
    // This constructor takes a string hostname or ip address and an int port
    // number
    //

      string host = a;
      int port = b;
      std->setHost(host);
      std->setPort(port);
      std->connectHost();
    }
  }

  string getResponse() {
  //
  // Returns a string containing the full text of the last response to a
  // query or command
  //

    return presponse;
  }

  int request(string request, string response) {
  //
  // Sends the requested string to the remote server and returns true if the
  // response string is contained in the response of the server
  //

    std->send(request + "\r\n");
    presponse = std->receive();
    if(pdebug == true) {
      write("%s\n%s",request,presponse);
    }
    if(search(presponse,response) == -1) {
      return false;
    } else {
      return true;
    } 
  }

  string getText() {
  //
  // Receives all available lines of text from the server until a . (period)
  // on a line by itself is reached. Returns the full text as a string
  //

    string temp;
    string text;
    while(false == 0) {
      temp = std->receive();
      if(temp == ".\r\n") break;
      text += temp;
    }
    return text;
  } 

  int authenticate(string username, string password) {
  //
  // Sends username and password and returns true on successful authentication.
  // Currently only support authinfo simple.
  //

    if(request("authinfo user " + username, "381")) {
      if(request("authinfo pass " + password, "281")) {
        return true;
      } else {
        perror = "Password '"+ password + "' failed.";
        setError(perror);
        return false;
      }
    } else {
      perror = "Username '" + username + "' failed.";
      setError(perror);
      return false;
    }
  }

  int previousArticle() {
  //
  // Sets the article pointer to the previous article and returns true on 
  // success.
  //

    if(request("last","223")) {
      return true;
    } else {
      setError("No previous article.");
      return false;
    }
  }

  int nextArticle() {
  //
  // Sets the article pointer to the next article and returns true on success.
  //

    if(request("next","223")) {
      return true;
    } else {
      setError("No next article.");
      return false;
    }
  }
  
  int quit() {
  //
  // Quits the NNTP session and disconnects. Returns true on success.
  //

    if(request("quit","205")) {
      std->disconnect();
      return true;
    } else {
      setError("Quit command failed.");
      return false;
    }
  }

  int selectGroup(object|string a) {
    if(objectp(a) && a->class_id == "news_group") {
    //
    // selectGroup(news_group group);
    //
    // Selects the appropiate group indicated by the news_group object.
    //
    
      object group = a;

      return selectGroup(group->getName());
    }

    if(stringp(a)) {
    //
    // selectGroup(string group);
    //
    // Selects the group indicated by the string.

      string group = a;
      if(request("group" + group,"211")) {
        pcurrentGroup = group;
        return true;
      } else {
        string error;
        error = "Selecting group '" + group + "' failed.";
        setError(error);
        return false;
      }
    }
    return false;
  }

  object listGroups(void|string|object a) {
    if(stringp(a)) {
    //
    // listGroups(string search);
    //
    // Returns a news_groups collection containing news_group objects for 
    // all groups on the server that match the server parameter.
    //

      string search = a;

      object groups = .news_groups.news_groups();
      if(request("list active " + search , "215")) {
        string temp;
        object group;
        while(false == 0) {
          group = .news_group.news_group();
          temp = std->receive();
          if(temp == ".\r\n") break;
          group->init(temp);
          groups->addGroup(group);
        }
      return groups;
      }
      string error;
      error = "List groups containing '" + search + "' failed.";
      setError(error);
      return groups;
    }

    if(objectp(a) && a->class_id == "news_date") {
    //
    // listGroups(news_date date);
    //
    // Returns a collection of news_group objects as returned from the 
    // newsgroups list.

      object date = a;

      if (request("newsgroups " + date->getNewOutput(), "211")) {
        string temp;
        object groups = .news_groups.news_groups();
        object group;
        while(false == 0) {
          group = .news_group.news_group();
          temp = std->receive();
          if(temp == ".\r\n") break;
          group->init(temp);
          groups->addGroup(group);
        }
        return groups;
      }
    }

    //
    // listGroups();
    //
    // Returns a collection of news_group objects as retrieved from the list
    // command.

    if (request("list","215")) {
      string temp;
      object groups = .news_groups.news_groups();
      object group;
      while(false == 0) {
        group = .news_group.news_group();
        temp = std->receive();
        if(temp == ".\r\n") break;
        group->init(temp);
        groups->addGroup(group);
      }
      return groups;
    }

    object groups = .news_groups.news_groups();
    setError("List command failed.");
    return groups;
  }

  object getOverview(int a, void|int b) {
    if(!b) {
    //
    // getOverview(int art);
    //
    // Returns the overview information for a single article based on 
    // article number.
    //

      int art = a;
      string tempstream;
      tempstream = sprintf("xover %d",art);
      if (request(tempstream,"224")) {
        string temp;
        object articles = .news_articles.news_articles();
        object article;

        while(false==0) {
          article = .news_article.news_article();
          temp = std->receive();
          if(temp == ".\r\n") break;
          article->init(temp);
          articles->addArticle(article);
        }
        return articles;
      }

      object articles = .news_articles.news_articles();
      string error;
      error = sprintf("Unable to get overview information for %d.",art);
      setError(error);
      return articles;
    }

    if(intp(a) && intp(b)) {
    //
    // getOverview(int first, int last);
    //
    // Returns a collection of news_article objects containing the overview
    // header information
    //
     
      int first = b;
      int last = a;
      string tempstream;

      tempstream = sprintf("xover %d-%d",first,last);
      if (request(tempstream,"224")) {
        string temp;
        object articles = .news_articles.news_articles();
        object article;
        while (false == 0) {
          article = .news_article.news_article();
          temp = std->receive();
          if(temp == ".\r\n") break;
          article->init(temp);
          articles->addArticle(article);
        }
        return articles;
      }
      object articles = .news_articles.news_articles();
      setError("Overview command failed.");
      return articles;
    }
  }

  object getBody(object|int|string a, object|void|string b) {
    if(stringp(a)) {
    //
    // getBody(string msg_id);
    //
    // Returns a news_article indicated by msg_id with the body set.
    //

      string msg_id = a;

      object article = .news_article.news_article();
      article->setId(msg_id);
      string temp; 
      temp=sprintf("body %d",msg_id);
      if(!request(temp,"222")) {
        string error;
        error = sprintf("Unable to retrieve body with ID %s.",msg_id);
        setError(error);
        return article;
      }
      article->setBody(getText());
      return article;
    }

    if(objectp(a) && a->class_id == "news_articles" 
                   && objectp(b) && b->class_id == "news_group") {
    //
    // getBody(news_articles articles, news_group group);
    //
    // Returns a news_articles collection containing the bodies of the
    // articles in the news_articles arguments from the group as
    // news_group group.

      object articles = a;
      object group = b;

      selectGroup(group->getName());
      return getBody(articles);
    }

    if(objectp(a) && a->class_id == "news_articles" && stringp(b)) {
    //
    // getBody(news_articles articles, string group);
    //
    // Returns a news_articles collection containing the bodies of the 
    // articles in the news_articles argument from the group passed as
    // group.
    //
    
       object articles = a; 
       string group = b;

       selectGroup(group);
       return getBody(articles);
    } 

    if(objectp(a) && a->class_id == "news_articles" && !stringp(b)) {
    //
    // getBody(news_articles articles); 
    //
    // Returns a news_articles collection containing the bodies of all of the
    // news_article objects contained in the articles collection.
    //
        object articles = a;

        object new_articles = .news_articles.news_articles();
        while(!articles->isEmpty()) {
          new_articles->addArticle(getBody(articles->getArticle()));
        }
        return new_articles;
    }

    if(objectp(a) && class_id == "news_article") {
    //
    // getBody(news_article article);
    //
    // Returns the news_article article with the body set.
    //

        object article = a;
        string temp;

        temp = sprintf("body %d",article->getNumber());
        if(!request(temp,"222")) {
          string error;
          error = sprintf("Unable to retrieve body %d.",article->getNumber());
          setError(error);
          return article;
        }
        article->setBody(getText());

        return article;
    }

    if(intp(a)) {
    // 
    // getBody(int article_no);
    //
    // Returns a news_article object containing the body of the article
    // indicated by the article number.
    //

      int article_no = a;
      object article = .news_article.news_article();

      article->setNumber(article_no);

      return getBody(article);
    }
  }

  object getArticle(object|int|string a, void|string|object b) {
    if(stringp(a)) {
    //
    // getArticle(string msg_id);
    //
    // Returns a news_article object containing the headers and body 
    // indicated by the message id.
    //

      string msg_id = a;
      object article = .news_article.news_article();
      article->setId(msg_id);
      string temp;
      string buffer;
      string header;
      string hvalue;
      int seperator;
      temp = sprintf("article %s",msg_id);
      if(!request(temp,"220")) {
        string error;
        error = sprintf("Unable to retrieve article with ID %s.",msg_id);
        setError(error);
        return article;
      } 

      while( false == 0) {
        buffer = std->receive();
        if(search(buffer,":") == -1) break;
        sscanf(buffer,"%s: %s\r\n",header, hvalue);
        article->setHeader(header,hvalue);
      }
      
      article->setBody(getText());
      return article;

    }

    if(objectp(a) && a->class_id == "news_article") {
    //
    // getArticle(news_article article);
    //
    // Returns the news_article article with all headers and the body set.
    //

      object article = a;

      string temp;
      string buffer;
      string header;
      string hvalue;
      int seperator;
      temp = sprintf("article %d",article->getNumber());
      if(!request(temp,"220")) {
        string error;
        error = sprintf("Unable to retrieve article %d.",article->getNumber());
        setError(error);
        return article;
      }
      
      while (false == 0) {
        buffer = std->receive();
        if(search(buffer,":") == -1) break;
        sscanf(buffer,"%s: %s\r\n",header, hvalue);
        article->setHeader(header,hvalue);
      }
      article->setBody(getText());
      return article;
    }

    if(objectp(a) && a->class_id == "news_articles" && stringp(b)) {
    //
    // getArticle(news_articles articles, string group);
    //
    // Returns a news_articles collection containing the article in the
    // news_articles arguments from the group passed as group.
    //
       object articles = a;
       string group = b;

       selectGroup(group);
       return getArticle(articles);
    }

    if(objectp(a) && a->class_id == "news_articles" && !stringp(b)) {
    //
    // getArticle(news_articles articles);
    //
    // Returns a news_articles collection with containing all headers and
    // bodies for each news_article in articles.
    //

      object articles = a;

      object new_articles = .news_articles.news_articles();
      while(!articles->isEmpty()) {
        new_articles->addArticle(getArticle(articles->getArticle()));
      }
      return new_articles;
    }

    if(objectp(a) && a->class_id == "news_articles"
                  && objectp(b) && b->class_id == "news_group") {
    //
    // getArticle(news_articles articles, news_group group);
    //
    // Returns a news_articles collection containing the articles in the
    // news_articles arguments from the group passwd as the news_group group.
    //
    
      object articles = a;
      object group = b;

      selectGroup(group->getName());
      return getArticle(articles);
    }

    if(intp(a)) {
    //
    // getArticle(int article_no);
    //
    // Returns a news_article object with the headers and body of the 
    // selected article.
    //

      int article_no = a;
      object article = .news_article.news_article();

      article->setNumber(article_no);

      return getArticle(article);
    }
  }

  object getHeader(int|object|string a, object|void|string b) {
    if(intp(a)) {
    //
    // getHeader(int article_no);
    //
    // Returns a news_article object containg the headers for the indicated
    // article.
  
      int article_no = a;
      object article = .news_article.news_article();

      article->setNumber(article_no);

      return getHeader(article);
    }

    if(stringp(a)) {
    //
    // getHeader(string msg_id);
    //
    // Returns a news_article indicated by msg_id which contains all of the 
    // article headers.
    //

      string msg_id = a;

      object article = .news_article.news_article();
      article->setId(msg_id);
      string temp;
      string buffer;
      string header;
      string hvalue;
      string seperator;
      temp = sprintf("article %d",msg_id);
      if(!request(temp,"220")) {
        string error;
        error = sprintf("Unable to retrieve headers with ID %s.",msg_id);
        setError(error);
        return article;
      }
 
      while (false == 0) {
        buffer = std->receive();
        if(search(buffer,":") == -1) break;
        sscanf(buffer,"%s: %s\r\n",header, hvalue);
        article->setHeader(header,hvalue);
      } 
      return article;
    }

    if(objectp(a) && a->class_id == "news_article") {
    // 
    // getHeader(news_article article);
    //
    // Returns the news_article article with all headers set.
    //

      object article = a;
      string temp;
      string buffer;
      string header;
      string hvalue;
      string seperator;

      temp = sprintf("article %d",article->getNumber());
      if(!request(temp,"220")) {
        string error;
        error = sprintf("Unable to retrieve headers %d",article->getNumber());
        setError(error);
        return article;
      }

      while (false == 0) {
        buffer = std->receive();
        if(search(buffer,":") == -1) break;
          sscanf(buffer,"%s: %s\r\n",header, hvalue);
          article->setHeader(header,hvalue);
      } 
      return article;
    }

    if(objectp(a) && a->class_id == "news_articles"
                   && objectp(b) && b->class_id == "news_group") {
    //
    // getHeader(news_articles articles, news_group group);
    //
    // Returns a news_articles collection containing the headers of the
    // articles arguments from the group passed as news_group group.
      
      object articles = a;
      object group = b;

      selectGroup(group->getName());
      return getHeader(articles);
    }

    if(objectp(a) && a->class_id == "news_articles" && stringp(b)) {
    //
    // getHeader(news_articles articles, string group);
    //
    // Returns a news_articles collection containing the headers of the 
    // articles in the news_articles argument from the group passed as group.
    //
    
      object articles = a;
      string group = b;

      selectGroup(group);
      return getHeader(articles);
    }

    if(objectp(a) && a->class_id == "news_articles" && !stringp(b)) {
    //
    // getHeader(news_articles articles);
    //
    // Returns a news_articles collection with all of the headers set for 
    // each article in articles.
    //

      object articles = a;

      object new_articles = .news_articles.news_articles();
      while(!articles->isEmpty()) {
        new_articles->addArticle(getHeader(articles->getArticle()));
      }
      return new_articles;
    }
  }

  object getGroup(object|string a) {
    if(stringp(a)) {
    // 
    // getGroup(string group);
    //
    // Returns a news_group object containing the current first and last
    // article numbers if the group exists. Otherwise returns an empty
    // news_group object.
    //
   
      string group = a;

      object groups = .news_groups.news_groups();
      object newgroup = .news_group.news_group();
      groups = listGroups(group);

      if(groups->count() == 1) {
        return groups->getGroup();
      }
      string error;
      error = "Unable to find group '" + group + "'.";
      setError(error);
      newgroup->setName(group);
      return newgroup;
    }

    if(objectp(a) && a->class_id == "news_group") {
    //
    // getGroup(news_group group);
    // 
    // Returns a news_group object containing the current first and last
    // article numbers if the group exists. Otherwise returns an empty
    // news_group object.

      object group = a;

      return getGroup(group->getName());
    }
  }

  int downloadBody(object|int a, void|string|object b) {
    if(objectp(a) && a->class_id == "news_articles" && !stringp(b)) {
    //
    // downloadBody(news_articles articles);
    //
    // Saves the bodies of all of the articles in news_articles to disk
    // using the article number preceeded by "A" for a file name. 
    // Returns the number of articles saved successfully.
    //

      object articles = a;

      int saved = 0;
      while(!articles->isEmpty()) {
        saved = saved + downloadBody(articles->getArticle());
      }
      return saved;
    }

    if(objectp(a) && a->class_id == "news_article" && !stringp(b)) {
    //
    // int downloadBody(news_article article);
    //
    // Saves the body of the article indicated by news_article to disk as
    // the file "A" followed by the article number.
    //

      object article = a;

      string temp;
      temp = sprintf("A%d",article->getNumber);
      return (downloadBody(article,temp));
    }

    if(objectp(a) && a->class_id == "news_article" && stringp(b)) {
    //
    // downloadBody(news_article article, string path);
    //
    // Saves the body of an article indicated by news_article to a file
    // indicated by path.

       object article = a;
       string path = b;

       string tempPath;
       if(puseGroupName) {
         tempPath = psavePath + pcurrentGroup + "/";
       } else {
         tempPath = psavePath;
       }
       path = tempPath + path;
       article->getBody(article);
       article->saveBody(path);
       return 1;
    }

    if(objectp(a) && a->class_id == "news_articles"
                   && stringp(b)) {
    //
    // downloadBody(news_articles articles, string group);
    //
    // Downloads the bodies of the articles contained in the news_articles
    // collection from the group indicated by the string group.
    //
  
      object articles = a;
      string group = b;

      selectGroup(group);
      return downloadBody(articles);
    }

    if(objectp(a) && a->class_id == "news_articles"
                   && objectp(b) && b->class_id == "news_group") {
    //
    // downloadBody(news_articles articles, news_group group);
    //
    // Downloads the bodies of the articles in the news_articles collection
    // from the group indicated by the news_group object group.
    //

      object articles = a;
      object group = b;

      selectGroup(group->getName());
      return downloadBody(articles);
    }

    if(intp(a)) {
    //
    // downloadBody(int art_no);
    //
    // Saves the body of the article indicated by article number to the 
    // current directory as art_no.art.

      int art_no = a;

      object article = .news_article.news_article();
      article->setNumber(art_no);
      return (downloadBody(article));
    }
  }

  int setSavePath(string path) {
  //
  // Set the path prefix for all saved files. This path must exist and end in 
  // a slash
  //

    psavePath = path;
    return true;
  }

  string getSavePath() {
  //
  // Returns a string containing the path prefix for all saved files
  //

    return psavePath;
  }

  int useGroupNamePrefix(int answer) {
  //
  // Determines whether to use the current group name as a prefix for saved
  // files.

    puseGroupName = answer;
    return true;
  }

  int downloadArticle(int|object a, void|string|object b) {
    if(objectp(a) && a->class_id == "news_articles" && !stringp(b)) {
    //
    // downloadArticle(news_articles articles);
    //
    // Saves each article in articles to the default path.
    //

      object articles = a;

      int saved = 0;
      while(!articles->isEmpty()) {
        saved = saved + downloadArticle(articles->getArticles());
      }
      return saved;
    }

    if(objectp(a) && a->class_id == "news_articles" && stringp(b)) {
    //
    // downloadArticle(news_articles articles, string group);
    //
    // Downloads the articles contained in the news_articles collection from
    // the group indicated by the string group.
    //
      
      object articles = a;
      string group = b;

      selectGroup(group);
      return downloadArticle(articles);
    }

    if(objectp(a) && a->class_id == "news_articles"
                     && objectp(b) && b->class_id == "news_group") {
    //
    // downloadArticles(news_articles articles, news_group group);
    //
    // Downloads the articles contained in the news_articles collection
    // from the group indicated by the news_group group;
    //

      object articles = a;
      object group = b;

      selectGroup(group->getName());
      return downloadArticle(articles);
    }

    if(objectp(a) && a->class_id == "news_article" && !stringp(b)) {
    //
    // downloadArticle(news_article article);
    //
    // Saves the header and body of article to default path.
    //

        object article = a;

        string temp;
        temp = sprintf("A%d",article->getNumber());
        return (downloadArticle(article,temp));
    }

    if(objectp(a) && a->class_id == "news_article" && stringp(b)) {
    //
    // downloadArticle(news_article article, string path);
    //
    // Saves the entire news_article indicated by article to the file 
    // indicated by path. Saves both the headers and body.
    //

      object article = a;
      string path = b;

      string tempPath;
      if(puseGroupName) {
        tempPath = psavePath + pcurrentGroup + "/";
      } else {
        tempPath = psavePath;
      }
      path = tempPath + path;
      article = getArticle(article);
      article->saveArticle(path);
      return 1;
    }

    if(intp(a)) {
    //
    // downloadArticle(int art_no);
    //
    // Saves the article indicated by art_no to the default path.
    //
      int art_no = a;

      object article = .news_article.news_article();
      article->setNumber(art_no);
      return (downloadArticle(article));
    }
  }

  object searchGroup(string header, int first, int last, string search) {
  //
  // Performs an xpat search on specified header with string search between
  // articles first and last. Returns a news_articles collection with the
  // article numbers and given header filled.
  //

    object articles = .news_articles.news_articles();
    object article;
    string temp;
    object date = .news_date.news_date();
    string buffer;
    int seperator;

    temp = sprintf("xpat %s %d-%d %s",header,first,last,search);
    if(!request(temp,"221")) {
      string error;
      error = sprintf("Search for %s in %s header failed.",search,header);
      setError(error);
      return articles;
    }

    while (false == 0) {
      int number;
      string hvalue;
      article = .news_article.news_article();

      buffer = std->receive();
      if(buffer == ".\r\n") break;
      sscanf(buffer,"%d %s\r\n",number,hvalue);
      article->setNumber(number);
      article->setHeader(header,hvalue);
      article->setDate(date);
      articles->addArticle(article);
    }

    return articles;
  }

  object findJpgs(object a, void|int b) {
    if(objectp(a) && a->class_id == "news_group") {
    //
    // findJpgs(news_group group);
    // 
    // Returns a news_articles collection containing news_article objects for
    // all articles in the given group which contain jpg in the subject header.
    //

      object group = a;

      group = getGroup(group->getName());
      selectGroup(group);
      object articles = .news_articles.news_articles();
      articles = searchGroup("Subject", group->getFirst(), group->getLast(),
                            "*.jpg*");
      return articles;     
    }

    if(objectp(a) && a->class_id == "news_group" && intp(b)) {
    //
    // findJpgs(news_group group, int number);
    //
    // Returns a news_articles collection containing news_article objects for
    // last number articles in the given group which contain jpg in the
    // subject header.
    //
  
       object group = a;
       int number = b;

       group = getGroup(group->getName());
       selectGroup(group);
       object articles = .news_articles.news_articles();
       articles = searchGroup("Subject", group->getLast() - number, 
                              group->getLast(), "*.jpg*");
       return articles;
    }
  }

  object findMpgs(object a, void|int b) {
    if(objectp(a) && a->class_id == "news_group") {
    //
    // findMpgs(news_group group);
    //
    // Returns a news_articles collection containing news_article objects for
    // all articles in the given group which contain mpg in the subject header.
    //
   
      object group = a;

      group = getGroup(group->getName());
      selectGroup(group);
      object articles = .news_articles.news_articles();
      articles = searchGroup("Subject", group->getFirst(), group->getLast(),
                             "*.mpg*");
      return articles;
    }

    if(objectp(a) && a->class_id == "news_group" && intp(b)) {
    //
    // findJpgs(news_group group, int number);
    //
    // Returns a news_articles collection containing news_article objects for
    // last number articles in the given group which contain jpg in the
    // subject header.
    //
  
       object group = a;
       int number = b;

       group = getGroup(group->getName());
       selectGroup(group);
       object articles = .news_articles.news_articles();
       articles = searchGroup("Subject", group->getLast() - number, 
                              group->getLast(), "*.mpg*");
       return articles;
    }
  }

  object findmp3s(object a, void|int b) {
    if(objectp(a) && a->class_id == "news_group") {
    //
    // findmp3s(news_group group);
    //
    // Returns a news_articles collection containing news_article objects for
    // all articles in the given group which contain mp3 in the subject header.
    //
   
      object group = a;

      group = getGroup(group->getName());
      selectGroup(group);
      object articles = .news_articles.news_articles();
      articles = searchGroup("Subject", group->getFirst(), group->getLast(),
                             "*.mp3*");
      return articles;
    }

    if(objectp(a) && a->class_id == "news_group" && intp(b)) {
    //
    // findJpgs(news_group group, int number);
    //
    // Returns a news_articles collection containing news_article objects for
    // last number articles in the given group which contain jpg in the
    // subject header.
    //
  
       object group = a;
       int number = b;

       group = getGroup(group->getName());
       selectGroup(group);
       object articles = .news_articles.news_articles();
       articles = searchGroup("Subject", group->getLast() - number, 
                              group->getLast(), "*.mp3*");
       return articles;
    }
  }

  void setDebug(int debug) {
  //
  // Turn debug on or off.
  //

    if(debug == true) pdebug = true;
    if(debug == false) pdebug = false;
  }

  void setError(string error) {
  //
  // Internal function to set the error message.
  //
  
    perror = error;
    if(pdebug == true) write("Error: %s\n",perror);
  }
};
